package cd.casic.ci.common.pipeline.utils;

import cd.casic.ci.common.pipeline.Model;
import cd.casic.ci.common.pipeline.container.*;
import cd.casic.ci.common.pipeline.enums.BuildStatus;
import cd.casic.ci.common.pipeline.enums.JobRunCondition;
import cd.casic.ci.common.pipeline.option.JobControlOption;
import cd.casic.ci.common.pipeline.pojo.element.Element;
import cd.casic.ci.common.pipeline.pojo.element.ElementAdditionalOptions;
import cd.casic.ci.common.pipeline.pojo.element.trigger.ManualTriggerElement;
import cd.casic.ci.common.pipeline.pojo.element.trigger.RemoteTriggerElement;
import cd.casic.framework.commons.util.reflect.ReflectUtil;
import cn.hutool.json.JSON;
import cn.hutool.json.JSONUtil;

import java.util.*;

import static cd.casic.ci.common.pipeline.pojo.element.ElementAdditionalOptions.RunCondition.*;

/**
 * @Author：mianbin
 * @Package：cd.casic.ci.common.pipeline.utils
 * @Project：ops-pro
 * @name：ModelUtils
 * @Date：2025/03/26 15:53
 * @Filename：ModelUtils
 * @description：Todo
 */
public class ModelUtils {
    public static void initContainerOldData(Container c) {
        if (c instanceof NormalContainer) {
            NormalContainer var = (NormalContainer) c;
            if (var.getJobControlOption() == null) {

                var.setJobControlOption(new JobControlOption(
                        true,
                        var.getMaxRunningMinutes(),
                        String.valueOf(var.getMaxRunningMinutes()),
                        var.getEnableSkip() ? (var.getConditions() != null && !var.getConditions().isEmpty() ? JobRunCondition.CUSTOM_VARIABLE_MATCH_NOT_RUN : JobRunCondition.STAGE_RUNNING) : JobRunCondition.STAGE_RUNNING,
                        var.getConditions()
                ));
            }
        } else if (c instanceof VMBuildContainer) {
            VMBuildContainer var = (VMBuildContainer) c;
            if (var.getJobControlOption() == null) {
                var.setJobControlOption(new JobControlOption(
                        true,
                        var.getMaxRunningMinutes(),
                        String.valueOf(var.getMaxRunningMinutes()),
                        JobRunCondition.STAGE_RUNNING,
                        null
                ));
            }
        }
    }

    public static boolean canManualStartup(TriggerContainer triggerContainer) {
        return triggerContainer.getElements().stream().anyMatch(element -> element instanceof ManualTriggerElement && ((ManualTriggerElement) element).elementEnabled());
    }

    public static boolean canRemoteStartup(TriggerContainer triggerContainer) {
        return triggerContainer.getElements().stream().anyMatch(element -> element instanceof RemoteTriggerElement && ((RemoteTriggerElement) element).elementEnabled());
    }

    public static boolean stageNeedPause(TriggerContainer triggerContainer) {
        return triggerContainer.getElements().stream().anyMatch(element -> element instanceof RemoteTriggerElement && ((RemoteTriggerElement) element).elementEnabled());
    }

    public static void refreshCanRetry(Model model) {
        List<Stage> stages = model.getStages();
        if (stages.isEmpty()) {
            return;
        }
        Stage lastStage = stages.get(stages.size() - 1);
        if (lastStage.getFinallyStage() && BuildStatus.parse(lastStage.getStatus()).isRunning()) {
            return;
        }
        for (Stage s : stages) {
            BuildStatus stageStatus = BuildStatus.parse(s.getStatus());
            s.setCanRetry(stageStatus.isFailure() || stageStatus.isCancel());
            for (Container c : s.getContainers()) {
                initContainerOldData(c);
                BuildStatus jobStatus = BuildStatus.parse(c.getStatus());
                c.setCanRetry(jobStatus.isFailure() || jobStatus.isCancel());
                if (c.getCanRetry() != null && c.getCanRetry()) {
                    refreshContainer(c);
                }
            }
        }
    }

    private static void refreshContainer(Container container) {
        var failElements = new ArrayList<Element>();
        container.getElements().forEach(element -> refreshElement(element, failElements));
    }

    private static void refreshElement(Element element, List<Element> failElements) {
        ElementAdditionalOptions additionalOptions = element.getAdditionalOptions();
        if (additionalOptions == null || !additionalOptions.isEnable()) {
            return;
        }
        var taskStatus = BuildStatus.parse(element.getStatus());
        if (!taskStatus.isFailure() && !taskStatus.isCancel()) {
            element.setCanRetry(null);
            element.setCanSkip(null);
            return;
        }
        element.setCanRetry(additionalOptions.isManualRetry());
        if (additionalOptions.isContinueWhenFailed()) {
            if (additionalOptions.getManualSkip()) {
                element.setCanSkip(true);
            } else {
                element.setCanRetry(null);
            }
        } else if (additionalOptions.getRunCondition() == PRE_TASK_FAILED_ONLY ||
                additionalOptions.getRunCondition() == PRE_TASK_FAILED_BUT_CANCEL ||
                additionalOptions.getRunCondition() == PRE_TASK_FAILED_EVEN_CANCEL) {
            element.setCanRetry(null);
            element.setCanSkip(null);
            failElements.forEach(e -> {
                e.setCanSkip(null);
                e.setCanRetry(null);
            });
        }
        if (element.getCanRetry()) {
            failElements.add(element);
        }
    }

    public static Model generatePipelineBuildModel(Map<String, Object> baseModelMap, Map<String, Object> modelFieldRecordMap) {
        JSON parse = JSONUtil.parse(generateBuildModelDetail(baseModelMap, modelFieldRecordMap));
        return JSONUtil.toBean(JSONUtil.toJsonStr(parse), Model.class);
    }

    public static Map<String, Object> generateBuildModelDetail(Map<String, Object> baseModelMap, Map<String, Object> modelFieldRecordMap) {
        modelFieldRecordMap.forEach((fieldRecordName, fieldRecordValue) -> {
            if (!ReflectUtil.isCollectionType(fieldRecordValue)) {
                baseModelMap.put(fieldRecordName, fieldRecordValue);
            } else if (baseModelMap.get(fieldRecordName) == null) {
                baseModelMap.put(fieldRecordName, fieldRecordValue);
            } else {
                if (fieldRecordValue instanceof Map<?, ?> && !((Map<?, ?>) fieldRecordValue).isEmpty()) {
                    var baseDataMap = (Map<String, Object>) baseModelMap.get(fieldRecordName);
                    var varDataMap = (Map<String, Object>) fieldRecordValue;
                    baseModelMap.put(fieldRecordName, generateBuildModelDetail(baseDataMap, varDataMap));
                } else if (fieldRecordValue instanceof List<?> && !((List<?>) fieldRecordValue).isEmpty()) {
                    var baseDataList = (List<Object>) baseModelMap.get(fieldRecordName);
                    var varDataList = (List<Object>) fieldRecordValue;
                    handleListFieldMergeBus(baseDataList, varDataList);
                }
            }
        });
        return baseModelMap;
    }

    private static void handleListFieldMergeBus(List<Object> baseDataList, List<Object> recordDataList) {
        for (int index = 0; index < recordDataList.size(); index++) {
            var listItemObj = recordDataList.get(index);
            if (!ReflectUtil.isCollectionType(listItemObj)) {
                if (index > baseDataList.size() - 1) {
                    baseDataList.add(listItemObj);
                } else {
                    baseDataList.set(index, listItemObj);
                }
            } else {
                if (listItemObj instanceof Map<?, ?> && !((Map<?, ?>) listItemObj).isEmpty()) {
                    var baseListItemDataMap = index > baseDataList.size() - 1 ? new HashMap<String, Object>() : (Map<String, Object>) baseDataList.get(index);
                    var varListItemDataMap = (Map<String, Object>) listItemObj;
                    baseDataList.set(index, generateBuildModelDetail(baseListItemDataMap, varListItemDataMap));
                } else if (listItemObj instanceof List<?> && !((List<?>) listItemObj).isEmpty()) {
                    var baseListItemDataList = index > baseDataList.size() - 1 ? new ArrayList<Object>() : (List<Object>) baseDataList.get(index);
                    var varListItemDataList = (List<Object>) listItemObj;
                    handleListFieldMergeBus(baseListItemDataList, varListItemDataList);
                }
            }
        }
    }

    public static Set<String> getModelAtoms(Model model) {
        var atomCodes = new HashSet<String>();
        for (var stage : model.getStages()) {
            for (Container container : stage.getContainers()) {
                for (Element element : container.getElements()) {
                    atomCodes.add(element.getAtomCode());
                }
            }
        }
        return atomCodes;
    }
}
